export { version } from "./version";
export { index } from "./embed";
export * as embed from "./embed";
export * as protocol from "./protocol";
export { logger, set_log_level } from "./core/logging";
export { settings } from "./core/settings";
export { default_resolver as Models } from "./base";
export { documents } from "./document";
export { safely } from "./safely";
//# sourceMappingURL=index.d.ts.map