export declare class Settings {
    private _dev;
    private _wireframe;
    private _force_webgl;
    private _force_fields;
    set dev(dev: boolean);
    get dev(): boolean;
    set wireframe(wireframe: boolean);
    get wireframe(): boolean;
    set force_webgl(force_webgl: boolean);
    get force_webgl(): boolean;
    set force_fields(force_fields: boolean);
    get force_fields(): boolean;
}
export declare const settings: Settings;
//# sourceMappingURL=settings.d.ts.map